/*
 * sock.c - Routines for manipulating sockets.
 */
#include <stdio.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <fcntl.h>

#include "global.h"
#include "sock.h"
#include "log.h"
#include "error.h"

/* This routine is so old I can't even remember writing it.  But I do
 * remember that it was an .h file because I didn't know putting code in a
 * header was bad magic yet.  anyway, this routine opens a connection to a
 * system and returns the fd.
 */

/*
 * Cleaned up some of the code to use memory routines which are now the
 * default. Also, the routine first checks to see if the address is in
 * dotted-decimal form before it does a name lookup. Finally, the newly
 * created socket is made non-blocking.
 *      - rjkaes
 */
int opensock(char *ip_addr, int port)
{
	int sock_fd, sockopt;
	struct sockaddr_in port_info;
	struct hostent *resolv_addy;

	if (!ip_addr) {
		log("opensock: ip_addr is NULL");
		tinyerr = ERR_INVALID;
		return -1;
	}
	if (port <= 0) {
		log("opensock: invalid port");
		tinyerr = ERR_INVALID;
		return -1;
	}

	memset((struct sockaddr*)&port_info, 0, sizeof(port_info));

	port_info.sin_family = AF_INET;

	/* First try looking up the address as aaa.bbb.ccc.ddd */
	if ((port_info.sin_addr.s_addr = inet_addr(ip_addr)) == -1) {
		/* Well, that didn't work... so try looking up the name */
		if ((resolv_addy = gethostbyname(ip_addr)) == NULL) {
			log("opensock: gethostbyname (%s)", strerror(errno));
			if (h_errno == TRY_AGAIN) {
				tinyerr = ERR_RETRY;
			} else {
				tinyerr = ERR_CONNERR;
			}
			return -1;
		}
		memcpy((char*)&port_info.sin_addr,
		       (char*)resolv_addy->h_addr_list[0],
		       resolv_addy->h_length);
	}

	port_info.sin_port = htons(port);

	if ((sock_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		log("opensock: socket (%s)", strerror(errno));
		tinyerr = ERR_CONNERR;
		return -1;
	}

	/* Make the socket non-blocking */
	sockopt = fcntl(sock_fd, F_GETFL);
	sockopt |= O_NONBLOCK;
	fcntl(sock_fd, F_SETFL, sockopt);

	if ((connect(sock_fd, (struct sockaddr*)&port_info, sizeof(port_info))) == -1) {
		if (errno == EINPROGRESS) {
			tinyerr = ERR_WOULDBLOCK;
			return sock_fd;
		} else {
			log("opensock: connect (%s)", strerror(errno));
			tinyerr = ERR_CONNERR;
		}
		return -1;
	}
	return sock_fd;
}

int setup_fd;
static struct sockaddr listen_sock_addr;

/*
 * Start listening to a socket.
 */
int init_listen_sock(int port)
{
	struct sockaddr_in laddr;
	int i=1;

	if (port <= 0) {
		log("init_listen_sock: invalid port");
		tinyerr = ERR_INVALID;
		return -1;
	}

	if ((setup_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		log("init_listen_sock: socket (%s)", strerror(errno));
		tinyerr = ERR_CONNERR;
		return -1;
	}

	if(setsockopt(setup_fd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) < 0) {
		log("init_listen_sock: setsockopt (%s)", strerror(errno));
		tinyerr = ERR_CONNERR;
		return -1;
	}
   
	memset(&listen_sock_addr, 0, sizeof(listen_sock_addr));
	memset(&laddr, 0, sizeof(laddr));
	laddr.sin_family = AF_INET;
	laddr.sin_port = htons(port);
	laddr.sin_addr.s_addr = inet_addr("0.0.0.0");
	if (bind(setup_fd, (struct sockaddr*)&laddr, sizeof(laddr)) < 0) {
		log("init_listen_sock: bind (%s)", strerror(errno));
		tinyerr = ERR_CONNERR;
		return -1;
	}
	if ((listen(setup_fd, MAXLISTEN)) != 0) {
		log("init_listen_sock: listen (%s)", strerror(errno));
		tinyerr = ERR_CONNERR;
		return -1;
	}

	return 0;
}

/*
 * Grab a pending connection
 */
int listen_sock(void)
{
	static int sock;
	int sz = sizeof(listen_sock_addr);

	if((sock = accept(setup_fd, &listen_sock_addr, &sz)) < 0) {
		log("listen_sock: accept (%s)", strerror(errno));
		tinyerr = ERR_CONNERR;
		return -1;
	}
	stats.num_listens++;

	return sock;
}

/*
 * Stop listening on a socket.
 */
void de_init_listen_sock(void)
{
	close(setup_fd);
}
