#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <sysexits.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>

#include "config.h"
#include "global.h"
#include "utils.h"
#include "log.h"
#include "error.h"

char *xstrdup(char *st)
{
	char *p;
	
	if (st == NULL) {
		log("xstrdup: NULL pointer");
		tinyerr = ERR_INVALID;
		return NULL;
	}
	
	if (!(p = strdup(st))) {
		log("xstrdup: out of memory (%s)", strerror(errno));
		tinyerr = ERR_NOMEM;
		return NULL;
	} else {
		return p;
	}
}

/*
 * for-sure malloc
 */
void *xmalloc(int sz)
{
	void *p;

	if (sz < 0) {
		log("xmalloc: trying to allocate a negative heap (%d)", sz);
		tinyerr = ERR_INVALID;
		return NULL;
	}

	if (!(p = malloc(sz))) {
		log("xmalloc: out of memory (%s)", strerror(errno));
		tinyerr = ERR_NOMEM;
		return NULL;
	}
	return p;
}

#ifdef USE_PROC
int calcload(void)
{
	char buf[BUFFER], *p;
	FILE *f;

	if (!config.cutoffload) {
		tinyerr = ERR_INVALID;
		return -1;
	}

	if(!(f=fopen("/proc/loadavg", "rt"))) {
		log("unable to read /proc/loadavg");
		config.cutoffload = 0.0;
		tinyerr = ERR_LOADING;
		return -1;
	}
	fgets(buf, BUFFER, f);
	p=strchr(buf, ' ');
	*p='\0';
	sscanf(buf, "%f", &load);
	fclose(f);
	return 0;
}
#else
int calcload(void)
{
	FILE *f;
	char buf[BUFFER];
	char *p, *y;

	if (!config.cutoffload) {
		tinyerr = ERR_INVALID;
		return -1;
	}

	if(!(f=popen(UPTIME_PATH, "r"))) {
		log("calcload: unable to exec uptime");
		config.cutoffload = 0.0;
		tinyerr = ERR_LOADING;
		return -1;
	}
	fgets(buf, BUFFER, f);
	p=strrchr(buf, ':');
	p+=2;
	y=strchr(p, ',');
	*y='\0';   
	sscanf(p, "%f", &load);
	pclose(f);
	return 0;
}
#endif

int showstats(int fd)
{
	static char *msg = 
"HTTP/1.0 200 OK\r\n\
Content-type: text/html\r\n\r\n\
<html><head><title>%s stats</title></head>\r\n\
<body>\r\n\
<center><h2>%s run-time statistics</h2></center><hr>\r\n\
<blockquote>\r\n\
Number of requests: %lu<br>\r\n\
Number of connections: %lu<br>\r\n\
Number of bad connections: %lu<br>\r\n\
Number of opens: %lu<br>\r\n\
Number of listens: %lu<br>\r\n\
Number of bytes (tx): %lu<br>\r\n\
Number of bytes (rx): %lu<br>\r\n
Number of garbage collects: %lu<br>\r\n\
Number of idle connection kills: %lu<br>\r\n\
Number of refused connections due to high load: %lu<br>\r\n\
Current system load average: %.2f (recalculated every %lu seconds)<br>\r\n\
</blockquote>\r\n\
</body></html>\r\n";
	char buf[BUFFER];

	if (fd < 0) {
		log("showstats: fd is not a socket (%d)", fd);
		tinyerr = ERR_INVALID;
		return -1;
	}

	sprintf(buf, msg, VERSION, VERSION, stats.num_reqs, stats.num_cons,
		stats.num_badcons, stats.num_opens, stats.num_listens,
		stats.num_tx, stats.num_rx, stats.num_garbage,
		stats.num_idles, stats.num_refused, load, LOAD_RECALCTIMER);
	send(fd, buf, strlen(buf), 0);
	return 0;
}
